/*! \mainpage Willkommen zum Mikrowebserver Projekt
 *
 * \section intro_sec Übersicht
 *
 * Zu Beginn der Aufstellung von Anforderungen an das uns gedanklich vorschwebende Projekt 
 * war eine Präzisierung der Funktionen des Microwebservers notwendig. Hier ging es vor allem 
 * um die vielseitige Nutzung des Gerätes, sowohl für Schüler, Studenten und Anfänger auf dem 
 * Gebiet der Programmierung als auch für fortgeschrittene Programmierer, Hobbyelektroniker 
 * und Dozenten in entsprechenden Unterrichtsfächern oder Kursen. Es sollte eine einfach zu 
 * bedienende und recht anschlussfreudige Entwicklungsumgebung für Schulungszwecke geschaffen werden, 
 * die aber auch im Hausgebrauch oder in der Industrie sinnvoll eingesetzt werden kann. 
 * Uns war aber auch klar, dass es schon eine Vielzahl von Mikrocontrollersystemen gab. Deshalb 
 * konzipierten wir ein System, welches den Ansprüchen an ein vollwertige Embbeded System gerecht wird, 
 * aber den Laien nicht mit Komplexität erschlägt. Folgende Funktionen sollte unser System vorweisen:
 *<br>
 *<br>
 *   * leicht zu programmierender Microcontroller <br>
 *   * kein Ausbau des Controllers aus der Hardware zum Programmieren (ISP)<br>
 *   * Programmieren in Assembler und C/C++ möglich<br>
 *   * vielfältige Schnittstellenverfügbarkeit wie RS232, SPI, USART, I2C(TWI), Netzwerk und Analogeingänge<br>
 *   * Debuggmodus (Industristandart JTAG) für Schulungs- und Entwicklungszwecke muss möglich sein<br>
 *   * hohe Taktrate des Systems (>10MHz)<br>
 *   * ausreichend Speicher (256kB Flash, 128kB RAM)<br>
 *   * kompakte Abmessungen der Platine<br>
 *   * wenige Bauelemente, geringe Beschaltung der IC`s<br>
 *   * großer Spannungsversorgungsbereich<br>
 *   * kostengünstige und leicht verfügbare Programmiersoftware und Programmiergeräte <br>
 *<br>
 * Dieses Projekt hat das Ziel, eine Entwicklungsumgebung für einen ATmega2561 mit einem
 * Netzwerkinterface zu erstellen, das mit Hilfe des ENC28j60 von Microchip realisiert wird. Ziel ist
 * es, eine Plattform für Entwickler zu bieten, der die wichtigsten Funktionen zur Verfügung 
 * stellt, so dass der Entwickler sich auf die eigentliche Applikation konzentrieren kann.
 */

/****************************************************************************
 *            main.c
 *
 *  Mon May 29 20:26:59 2007
 *  Copyright  2007  Dirk Broßwick
 *  Email sharandac(at)snafu.de
 ****************************************************************************/
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */ 


#include "system/init.h"

#include "apps/telnet/telnet.h"
#include "apps/httpd/httpd2.h"

#ifdef __AVR_ATmega2561__
	#include "apps/mp3-streamingclient/mp3-streaming.h"
#endif
#include "config.h"

int main( void ) 
{
	init();
		
	// Dienste starten
	#ifdef TELNETSERVER
		telnet_init();
	#endif
	#ifdef HTTPSERVER
		httpd_init();
	#endif
	#ifdef __AVR_ATmega2561__
		mp3client_init();
	#endif
	
	// die mainloop, hier alles eintragen, was sich dienst nennt
	while(1)
	{	
		#ifdef TELNETSERVER
			telnet_thread();
		#endif
		#ifdef HTTPSERVER
			httpd_thread();
		#endif
		#ifdef __AVR_ATmega2561__
			mp3client_thread();
		#endif
	}
}
