/*! \file stdout.c \brief STDOUT-Funktion um die Ausgaben umzulenken */
//***************************************************************************
//*            stdout.c
//*
//*  Sat July  13 21:07:42 2008
//*  Copyright  2008  Dirk Broßwick
//*  Email: sharandac@snafu.de
//****************************************************************************/
///	\ingroup system
///	\defgroup stdout Stdout Funktionen (stdout.c)
///	\code #include "stdout.h" \endcode
///	\par Uebersicht
//****************************************************************************/
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */
//@{
#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <stdlib.h>
#include "stdout.h"
#include "hardware/uart/uart.h"
#include "system/net/tcp.h"

#include "config.h"


static unsigned char BUFFER[ MAX_TCP_Datalenght ];
 
struct STDOUT streamout;

// stdout auf UART_Send_Byte verbiegen
static FILE mystdout = FDEV_SETUP_STREAM( STDOUT_Send_Byte , STDOUT_Get_Byte , _FDEV_SETUP_RW );


/*------------------------------------------------------------------------------------------------------------*/
/*!\brief Initialisiert die Standart Out.
 * \param 	void		NONE
 * \return	void		NONE
 */
/*------------------------------------------------------------------------------------------------------------*/
void STDOUT_INIT( void )
{
	// printf auf uart umbiegen
	stdout = &mystdout;
	streamout.TYPE = UNKNOWN;
	streamout.BUFFER = BUFFER ;
	streamout.BUFFER_POS = 0 ;
	streamout.XPOS = 1;
	streamout.YPOS = 1;
}	

/*------------------------------------------------------------------------------------------------------------*/
/*!\brief Sendet ein Byte auf der Standart Out.
/*------------------------------------------------------------------------------------------------------------*/
int STDOUT_Send_Byte ( char Byte, FILE * stream )
{
	switch ( streamout.TYPE )
	{
		
		case RS232:		UART_Send_Byte ( Byte );
						break;		
		#ifdef TCP
		case _TCP:		if ( streamout.BUFFER_POS == MAX_TCP_Datalenght )
							STDOUT_Flush();
		
						streamout.BUFFER[ streamout.BUFFER_POS++ ] = Byte;
						break;
		#endif
		default:		break;
	}
}

/*------------------------------------------------------------------------------------------------------------*/
/*!\brief Holt ein Byte von der Standart Out.
/*------------------------------------------------------------------------------------------------------------*/
int STDOUT_Get_Byte ( FILE * stream )
{
	return( EOF );
}

/*------------------------------------------------------------------------------------------------------------*/
/*!\brief Setzt eine neue Standart Out.
 * \param 	TYPE		Typ auf den umgebogen werden soll.
 * \param 	DEVICE		Devicenummer oder Socket auf den umgebogen werden soll.
 * \return	int			0 alles Okay. Fehler gleich -1.
 */
/*------------------------------------------------------------------------------------------------------------*/
int STDOUT_set( char TYPE, int DEVICE )
{
	STDOUT_Flush();
	
	streamout.TYPE = TYPE;
	streamout.DEVICE = DEVICE;
	streamout.BUFFER_POS = 0;
	
}

/*------------------------------------------------------------------------------------------------------------*/
/*!\brief Sichert die aktuelle Standart Out.
 * \param 	pStruct		Pointer auf Struktur vom Typ streamout in den der aktuelle Standart Out gesichert werden soll.
 * \return	int			0 alles Okay.
 */
/*------------------------------------------------------------------------------------------------------------*/
int STDOUT_save( void * pStruct )
{
	STDOUT_Flush();
	memcpy( pStruct, &streamout, sizeof( streamout ) );
	return( 0 );
}

/*------------------------------------------------------------------------------------------------------------*/
/*!\brief Stelle den gesicherten Standart Out wieder her.
 * \param 	pStruct		Pointer auf Struktur vom Typ streamout in den der wieder hergestellt werden soll.
 * \return	int			0 alles Okay.
 */
/*------------------------------------------------------------------------------------------------------------*/
int STDOUT_restore( void * pStruct )
{
	STDOUT_Flush();
	memcpy( &streamout, pStruct, sizeof( streamout ) );
	return( 0 );
}

/*------------------------------------------------------------------------------------------------------------*/
/*!\brief Schickt alles Daten die sich um Puffer befinden könnten raus.
 * \param 	void		NONE
 * \return	void		NONE
 */
/*------------------------------------------------------------------------------------------------------------*/
void STDOUT_Flush( void )
{
	switch( streamout.TYPE )
	{
		case	RS232:		break;
#ifdef TCP
		case	_TCP:		if ( streamout.BUFFER_POS != 0 )
								PutSocketData_RPE ( streamout.DEVICE, streamout.BUFFER_POS, streamout.BUFFER, RAM );
							streamout.BUFFER_POS = 0;
							break;
#endif
		default:			break;
	}
}
//@}
